@@ -27,6 +27,7 @@
 """
 
 import logging
+import os
 
 from plainbox import __version__ as version
 from plainbox.impl.applogic import PlainBoxConfig
@@ -34,6 +35,7 @@ from plainbox.impl.commands import PlainBoxToolBase
 from plainbox.impl.commands.check_config import CheckConfigCommand
 from plainbox.impl.commands.dev import DevCommand
 from plainbox.impl.commands.run import RunCommand
+from plainbox.impl.commands.startprovider import StartProviderCommand
 from plainbox.impl.commands.selftest import SelfTestCommand
 from plainbox.impl.logging import setup_logging
 
@@ -46,16 +48,6 @@ class PlainBoxTool(PlainBoxToolBase):
     Command line interface to PlainBox
     """
 
-    @classmethod
-    def get_config_cls(cls):
-        """
-        Get the Config class that is used by this implementation.
-
-        This can be overriden by subclasses to use a different config class
-        that is suitable for the particular application.
-        """
-        return PlainBoxConfig
-
     @classmethod
     def get_exec_name(cls):
         """
@@ -74,14 +66,33 @@ class PlainBoxTool(PlainBoxToolBase):
         """
         Add top-level subcommands to the argument parser.
 
-        This can be overriden by subclasses to use a different set of
+        This can be overridden by subclasses to use a different set of
         top-level subcommands.
         """
         # TODO: switch to plainbox plugins
-        RunCommand(self._provider_list, self._config).register_parser(subparsers)
+        RunCommand(self._provider_list, self._config).register_parser(
+            subparsers)
         SelfTestCommand().register_parser(subparsers)
         CheckConfigCommand(self._config).register_parser(subparsers)
-        DevCommand(self._provider_list, self._config).register_parser(subparsers)
+        DevCommand(self._provider_list, self._config).register_parser(
+            subparsers)
+        StartProviderCommand().register_parser(subparsers)
+
+    @classmethod
+    def get_config_cls(cls):
+        """
+        Get the Config class that is used by this implementation.
+
+        This can be overridden by subclasses to use a different config
+        class that is suitable for the particular application.
+        """
+        return PlainBoxConfig
+
+    def get_gettext_domain(self):
+        return "plainbox"
+
+    def get_locale_dir(self):
+        return os.getenv("PLAINBOX_LOCALE_DIR", None)
 
 
 def main(argv=None):
