@@ -5,9 +5,9 @@
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU General Public License version 3,
+# as published by the Free Software Foundation.
+
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,6 +34,7 @@ import base64
 
 import pkg_resources
 
+from plainbox.i18n import gettext as _
 
 logger = getLogger("plainbox.exporter")
 
@@ -75,6 +76,8 @@ class SessionStateExporterBase(metaclass=ABCMeta):
     OPTION_WITH_JOB_DEFS = 'with-job-defs'
     OPTION_WITH_ATTACHMENTS = 'with-attachments'
     OPTION_WITH_COMMENTS = 'with-comments'
+    OPTION_WITH_JOB_VIA = 'with-job-via'
+    OPTION_WITH_JOB_HASH = 'with-job-hash'
 
     SUPPORTED_OPTION_LIST = (
         OPTION_WITH_IO_LOG,
@@ -86,6 +89,8 @@ class SessionStateExporterBase(metaclass=ABCMeta):
         OPTION_WITH_JOB_DEFS,
         OPTION_WITH_ATTACHMENTS,
         OPTION_WITH_COMMENTS,
+        OPTION_WITH_JOB_VIA,
+        OPTION_WITH_JOB_HASH,
     )
 
     def __init__(self, option_list=None):
@@ -110,7 +115,7 @@ class SessionStateExporterBase(metaclass=ABCMeta):
         The subset of the data that should be saved may depend on a particular
         saver class and options selected by the user.
 
-        Must return a collection that can be handled by save_data().  Special
+        Must return a collection that can be handled by :meth:`dump()`.  Special
         care must be taken when processing io_log (and in the future,
         attachments) as those can be arbitrarily large.
         """
@@ -144,10 +149,22 @@ class SessionStateExporterBase(metaclass=ABCMeta):
                 continue
             data['result_map'][job_name] = OrderedDict()
             data['result_map'][job_name]['outcome'] = job_state.result.outcome
+            if job_state.result.execution_duration:
+                data['result_map'][job_name]['execution_duration'] = \
+                    job_state.result.execution_duration
             if self.OPTION_WITH_COMMENTS in self._option_list:
                 data['result_map'][job_name]['comments'] = \
                     job_state.result.comments
 
+            # Add Parent hash if requested
+            if self.OPTION_WITH_JOB_VIA in self._option_list:
+                data['result_map'][job_name]['via'] = \
+                    job_state.job.via
+
+            # Add Job hash if requested
+            if self.OPTION_WITH_JOB_HASH in self._option_list:
+                data['result_map'][job_name]['hash'] = job_state.job.checksum
+
             # Add Job definitions if requested
             if self.OPTION_WITH_JOB_DEFS in self._option_list:
                 for prop in ('plugin',
@@ -156,17 +173,17 @@ class SessionStateExporterBase(metaclass=ABCMeta):
                              'command',
                              'description',
                              ):
-                    if not getattr(job_state.result.job, prop):
+                    if not getattr(job_state.job, prop):
                         continue
                     data['result_map'][job_name][prop] = getattr(
-                        job_state.result.job, prop)
+                        job_state.job, prop)
 
             # Add Attachments if requested
-            if job_state.result.job.plugin == 'attachment':
+            if job_state.job.plugin == 'attachment':
                 if self.OPTION_WITH_ATTACHMENTS in self._option_list:
                     raw_bytes = b''.join(
                         (record[2] for record in
-                         job_state.result.io_log if record[1] == 'stdout'))
+                         job_state.result.get_io_log() if record[1] == 'stdout'))
                     data['attachment_map'][job_name] = \
                         base64.standard_b64encode(raw_bytes).decode('ASCII')
                 continue  # Don't add attachments IO logs to the result_map
@@ -177,12 +194,12 @@ class SessionStateExporterBase(metaclass=ABCMeta):
                 # saved, discarding stream name and the relative timestamp.
                 if self.OPTION_SQUASH_IO_LOG in self._option_list:
                     io_log_data = self._squash_io_log(
-                        job_state.result.io_log)
+                        job_state.result.get_io_log())
                 elif self.OPTION_FLATTEN_IO_LOG in self._option_list:
                     io_log_data = self._flatten_io_log(
-                        job_state.result.io_log)
+                        job_state.result.get_io_log())
                 else:
-                    io_log_data = self._io_log(job_state.result.io_log)
+                    io_log_data = self._io_log(job_state.result.get_io_log())
                 data['result_map'][job_name]['io_log'] = io_log_data
         return data
 
@@ -274,8 +291,10 @@ def get_all_exporters():
     for entry_point in sorted(iterator, key=lambda ep: ep.name):
         try:
             exporter_cls = entry_point.load()
+        except pkg_resources.DistributionNotFound as exc:
+            logger.info(_("Unable to load %s: %s"), entry_point, exc)
         except ImportError as exc:
-            logger.exception("Unable to import {}: {}", entry_point, exc)
+            logger.exception(_("Unable to import %s: %s"), entry_point, exc)
         else:
             exporter_map[entry_point.name] = exporter_cls
     return exporter_map
