@@ -25,9 +25,11 @@ Test definitions for plainbox.impl.box module
 """
 
 from inspect import cleandoc
+from io import TextIOWrapper
 from unittest import TestCase
 
 from plainbox import __version__ as version
+from plainbox.abc import IProvider1
 from plainbox.impl.box import main
 from plainbox.impl.commands.checkbox import CheckBoxInvocationMixIn
 from plainbox.impl.testing_utils import MockJobDefinition, suppress_warnings
@@ -35,18 +37,40 @@ from plainbox.testing_utils.io import TestIO
 from plainbox.vendor.mock import Mock
 
 
+def mock_whitelist(name, text, filename):
+    """
+    Create a mocked whitelist for
+    CheckBoxInvocationMixIn._get_matching_job_list(). Specifically
+    for ``ns.whitelists`` as passed to that function.
+
+    :param name:
+        Name of the mocked object, helps in debugging
+    :param text:
+        Full text of the whitelist
+    :param filename:
+        Filename of the whitelist file
+    """
+    whitelist = Mock(spec=TextIOWrapper, name=name)
+    whitelist.name = filename
+    whitelist.read.return_value = text
+    return whitelist
+
+
 class MiscTests(TestCase):
 
     def setUp(self):
-        self.job_foo = MockJobDefinition(name='foo')
-        self.job_bar = MockJobDefinition(name='bar')
-        self.job_baz = MockJobDefinition(name='baz')
-        self.obj = CheckBoxInvocationMixIn(Mock(name="checkbox"))
+        self.job_foo = MockJobDefinition(id='foo')
+        self.job_bar = MockJobDefinition(id='bar')
+        self.job_baz = MockJobDefinition(id='baz')
+        self.provider1 = Mock(IProvider1)
+        self.provider1.get_builtin_whitelists.return_value = []
+        self.provider_list = [self.provider1]
+        self.obj = CheckBoxInvocationMixIn(self.provider_list)
 
     def test_matching_job_list(self):
         # Nothing gets selected automatically
-        ns = Mock()
-        ns.whitelist = None
+        ns = Mock(name="ns")
+        ns.whitelist = []
         ns.include_pattern_list = []
         ns.exclude_pattern_list = []
         observed = self.obj._get_matching_job_list(ns, [
@@ -55,8 +79,8 @@ class MiscTests(TestCase):
 
     def test_matching_job_list_including(self):
         # Including jobs with glob pattern works
-        ns = Mock()
-        ns.whitelist = None
+        ns = Mock(name="ns")
+        ns.whitelist = []
         ns.include_pattern_list = ['f.+']
         ns.exclude_pattern_list = []
         observed = self.obj._get_matching_job_list(ns, [
@@ -65,8 +89,8 @@ class MiscTests(TestCase):
 
     def test_matching_job_list_excluding(self):
         # Excluding jobs with glob pattern works
-        ns = Mock()
-        ns.whitelist = None
+        ns = Mock(name="ns")
+        ns.whitelist = []
         ns.include_pattern_list = ['.+']
         ns.exclude_pattern_list = ['f.+']
         observed = self.obj._get_matching_job_list(ns, [
@@ -76,11 +100,9 @@ class MiscTests(TestCase):
     def test_matching_job_list_whitelist(self):
         # whitelists contain list of include patterns
         # that are read and interpreted as usual
-        whitelist = Mock()
-        whitelist.readlines.return_value = ['foo']
-        whitelists = [whitelist]
-        ns = Mock()
-        ns.whitelist = whitelists
+        ns = Mock(name="ns")
+        ns.whitelist = [
+            mock_whitelist("foo_whitelist", "foo", "foo.whitelist")]
         ns.include_pattern_list = []
         ns.exclude_pattern_list = []
         observed = self.obj._get_matching_job_list(ns, [
@@ -88,13 +110,11 @@ class MiscTests(TestCase):
         self.assertEqual(observed, [self.job_foo])
 
     def test_matching_job_list_multiple_whitelists(self):
-        whitelist_a = Mock()
-        whitelist_a.readlines.return_value = ['foo']
-        whitelist_b = Mock()
-        whitelist_b.readlines.return_value = ['baz']
-        whitelists = [whitelist_a, whitelist_b]
-        ns = Mock()
-        ns.whitelist = whitelists
+        ns = Mock(name="ns")
+        ns.whitelist = [
+            mock_whitelist("whitelist_a", "foo", "a.whitelist"),
+            mock_whitelist("whitelist_b", "baz", "b.whitelist"),
+        ]
         ns.include_pattern_list = []
         ns.exclude_pattern_list = []
         observed = self.obj._get_matching_job_list(ns, [
@@ -103,8 +123,11 @@ class MiscTests(TestCase):
 
     def test_no_prefix_matching_including(self):
         # Include patterns should only match whole job name
-        ns = Mock()
-        ns.whitelist = None
+        ns = Mock(name="ns")
+        ns.whitelist = [
+            mock_whitelist("whitelist_a", "fo", "a.whitelist"),
+            mock_whitelist("whitelist_b", "ba.+", "b.whitelist"),
+        ]
         ns.include_pattern_list = ['fo', 'ba.+']
         ns.exclude_pattern_list = []
         observed = self.obj._get_matching_job_list(ns, [self.job_foo,
@@ -113,17 +136,17 @@ class MiscTests(TestCase):
 
     def test_no_prefix_matching_excluding(self):
         # Exclude patterns should only match whole job name
-        ns = Mock()
-        ns.whitelist = None
+        ns = Mock(name="ns")
+        ns.whitelist = []
         ns.include_pattern_list = ['.+']
         ns.exclude_pattern_list = ['fo', 'ba.+']
-        observed = self.obj._get_matching_job_list(ns, [self.job_foo,
-                                                        self.job_bar])
+        observed = self.obj._get_matching_job_list(
+            ns, [self.job_foo, self.job_bar])
         self.assertEqual(observed, [self.job_foo])
 
     def test_invalid_pattern_including(self):
-        ns = Mock()
-        ns.whitelist = None
+        ns = Mock(name="ns")
+        ns.whitelist = []
         ns.include_pattern_list = ['?']
         ns.exclude_pattern_list = []
         observed = self.obj._get_matching_job_list(
@@ -131,8 +154,8 @@ class MiscTests(TestCase):
         self.assertEqual(observed, [])
 
     def test_invalid_pattern_excluding(self):
-        ns = Mock()
-        ns.whitelist = None
+        ns = Mock(name="ns")
+        ns.whitelist = []
         ns.include_pattern_list = ['fo.*']
         ns.exclude_pattern_list = ['[bar']
         observed = self.obj._get_matching_job_list(
@@ -160,22 +183,25 @@ class TestMain(TestCase):
         self.assertEqual(call.exception.args, (0,))
         self.maxDiff = None
         expected = """
-        usage: plainbox [-h] [--version] [-c {src,deb,auto,stub,ihv}] [-v] [-D] [-C]
+        usage: plainbox [-h] [--version] [--providers {all,stub}] [-v] [-D] [-C]
                         [-T LOGGER] [-P] [-I]
-                        {run,self-test,check-config,dev} ...
+                        {run,self-test,check-config,dev,startprovider} ...
 
         positional arguments:
-          {run,self-test,check-config,dev}
+          {run,self-test,check-config,dev,startprovider}
             run                 run a test job
-            self-test           run integration tests
+            self-test           run unit and integration tests
             check-config        check and display plainbox configuration
             dev                 development commands
+            startprovider       create a new provider (directory)
 
         optional arguments:
           -h, --help            show this help message and exit
           --version             show program's version number and exit
-          -c {src,deb,auto,stub,ihv}, --checkbox {src,deb,auto,stub,ihv}
-                                where to find the installation of CheckBox.
+
+        provider list and development:
+          --providers {all,stub}
+                                which providers to load
 
         logging and debugging:
           -v, --verbose         be more verbose (same as --log-level=INFO)
@@ -197,9 +223,9 @@ class TestMain(TestCase):
                 main([])
             self.assertEqual(call.exception.args, (2,))
         expected = """
-        usage: plainbox [-h] [--version] [-c {src,deb,auto,stub,ihv}] [-v] [-D] [-C]
+        usage: plainbox [-h] [--version] [--providers {all,stub}] [-v] [-D] [-C]
                         [-T LOGGER] [-P] [-I]
-                        {run,self-test,check-config,dev} ...
+                        {run,self-test,check-config,dev,startprovider} ...
         plainbox: error: too few arguments
         """
         self.assertEqual(io.combined, cleandoc(expected) + "\n")
@@ -219,23 +245,21 @@ class TestSpecial(TestCase):
 
         optional arguments:
           -h, --help            show this help message and exit
-          -j, --list-jobs       List jobs instead of running them
+          -j, --list-jobs       list jobs instead of running them
           -J, --list-job-hashes
-                                List jobs with hashes instead of running them
+                                list jobs with cheksums instead of running them
           -e, --list-expressions
-                                List all unique resource expressions
-          -d, --dot             Print a graph of jobs instead of running them
-          --dot-resources       Render resource relationships (for --dot)
+                                list all unique resource expressions
+          -d, --dot             print a graph of jobs instead of running them
+          --dot-resources       show resource relationships (for --dot)
 
         job definition options:
           -i PATTERN, --include-pattern PATTERN
-                                Run jobs matching the given regular expression.
-                                Matches from the start to the end of the line.
+                                include jobs matching the given regular expression
           -x PATTERN, --exclude-pattern PATTERN
-                                Do not run jobs matching the given regular expression.
-                                Matches from the start to the end of the line.
+                                exclude jobs matching the given regular expression
           -w WHITELIST, --whitelist WHITELIST
-                                Load whitelist containing run patterns
+                                load whitelist containing run patterns
         """
         self.assertEqual(io.combined, cleandoc(expected) + "\n")
 
@@ -254,24 +278,30 @@ class TestSpecial(TestCase):
     def test_run_list_jobs(self):
         with TestIO() as io:
             with self.assertRaises(SystemExit) as call:
-                main(['-c', 'stub', 'dev', 'special', '--list-jobs'])
+                main(['--providers', 'stub', 'dev', 'special', '--list-jobs'])
             self.assertEqual(call.exception.args, (0,))
-        self.assertIn("stub/false", io.stdout.splitlines())
-        self.assertIn("stub/true", io.stdout.splitlines())
+        self.assertIn(
+            "2013.com.canonical.plainbox::stub/false", io.stdout.splitlines())
+        self.assertIn(
+            "2013.com.canonical.plainbox::stub/true", io.stdout.splitlines())
 
     def test_run_list_jobs_with_filtering(self):
         with TestIO() as io:
             with self.assertRaises(SystemExit) as call:
-                main(['-c', 'stub', 'dev', 'special',
-                      '--include-pattern=stub/false', '--list-jobs'])
+                main(['--providers', 'stub', 'dev', 'special',
+                      ('--include-pattern='
+                       '2013.com.canonical.plainbox::stub/false'),
+                      '--list-jobs'])
             self.assertEqual(call.exception.args, (0,))
-        self.assertIn("stub/false", io.stdout.splitlines())
-        self.assertNotIn("stub/true", io.stdout.splitlines())
+        self.assertIn(
+            "2013.com.canonical.plainbox::stub/false", io.stdout.splitlines())
+        self.assertNotIn(
+            "2013.com.canonical.plainbox::stub/true", io.stdout.splitlines())
 
     def test_run_list_expressions(self):
         with TestIO() as io:
             with self.assertRaises(SystemExit) as call:
-                main(['-c', 'stub', 'dev', 'special', '--list-expressions'])
+                main(['--providers', 'stub', 'dev', 'special', '--list-expressions'])
             self.assertEqual(call.exception.args, (0,))
         self.assertIn(
             'stub_package.name == "checkbox"', io.stdout.splitlines())
@@ -279,23 +309,25 @@ class TestSpecial(TestCase):
     def test_run_dot(self):
         with TestIO() as io:
             with self.assertRaises(SystemExit) as call:
-                main(['-c', 'stub', 'dev', 'special', '--dot'])
+                main(['--providers', 'stub', 'dev', 'special', '--dot'])
             self.assertEqual(call.exception.args, (0,))
         self.assertIn(
-            '\t"stub/true" [];', io.stdout.splitlines())
+            '\t"2013.com.canonical.plainbox::stub/true" [];',
+            io.stdout.splitlines())
         # Do basic graph checks
         self._check_digraph_sanity(io)
 
     def test_run_dot_with_resources(self):
         with TestIO() as io:
             with self.assertRaises(SystemExit) as call:
-                main(['-c', 'stub', 'dev', 'special', '--dot',
+                main(['--providers', 'stub', 'dev', 'special', '--dot',
                       '--dot-resources'])
             self.assertEqual(call.exception.args, (0,))
         self.assertIn(
-            '\t"stub/true" [];', io.stdout.splitlines())
+            '\t"2013.com.canonical.plainbox::stub/true" [];', io.stdout.splitlines())
         self.assertIn(
-            ('\t"stub/requirement/good" -> "stub_package" [style=dashed, label'
+            ('\t"2013.com.canonical.plainbox::stub/requirement/good" -> '
+             '"2013.com.canonical.plainbox::stub_package" [style=dashed, label'
              '="stub_package.name == \'checkbox\'"];'),
             io.stdout.splitlines())
         # Do basic graph checks
